@@ -43,11 +43,9 @@ class Bot(TelegramObject):
 
     Args:
         token (str): Bot's unique authentication.
-        **kwargs: Arbitrary keyword arguments.
-
-    Keyword Args:
         base_url (Optional[str]): Telegram Bot API service URL.
         base_file_url (Optional[str]): Telegram Bot API file URL.
+
     """
 
     def __init__(self,
@@ -141,8 +139,7 @@ class Bot(TelegramObject):
                     data['reply_markup'] = reply_markup
 
             result = request.post(url, data,
-                                  timeout=kwargs.get('timeout'),
-                                  network_delay=kwargs.get('network_delay'))
+                                  timeout=kwargs.get('timeout'))
 
             if result is True:
                 return result
@@ -208,10 +205,6 @@ class Bot(TelegramObject):
                 keyboard or to force a reply from the user.
             timeout (Optional[float]): If this value is specified, use it as
                 the definitive timeout (in seconds) for urlopen() operations.
-            network_delay (Optional[float]): If using the timeout (which is
-                a `timeout` for the Telegram servers operation),
-                then `network_delay` as an extra delay (in seconds) to
-                compensate for network latency. Defaults to 2.
 
         Returns:
             :class:`telegram.Message`: On success, the sent message is
@@ -258,10 +251,6 @@ class Bot(TelegramObject):
                 receive a notification with no sound.
             timeout (Optional[float]): If this value is specified, use it as
                 the definitive timeout (in seconds) for urlopen() operations.
-            network_delay (Optional[float]): If using the timeout (which is
-                a `timeout` for the Telegram servers operation),
-                then `network_delay` as an extra delay (in seconds) to
-                compensate for network latency. Defaults to 2.
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the
@@ -317,10 +306,6 @@ class Bot(TelegramObject):
                 keyboard or to force a reply from the user.
             timeout (Optional[float]): If this value is specified, use it as
                 the definitive timeout (in seconds) for urlopen() operations.
-            network_delay (Optional[float]): If using the timeout (which is
-                a `timeout` for the Telegram servers operation),
-                then `network_delay` as an extra delay (in seconds) to
-                compensate for network latency. Defaults to 2.
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the
@@ -387,10 +372,6 @@ class Bot(TelegramObject):
                 keyboard or to force a reply from the user.
             timeout (Optional[float]): If this value is specified, use it as
                 the definitive timeout (in seconds) for urlopen() operations.
-            network_delay (Optional[float]): If using the timeout (which is
-                a `timeout` for the Telegram servers operation),
-                then `network_delay` as an extra delay (in seconds) to
-                compensate for network latency. Defaults to 2.
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the
@@ -451,10 +432,6 @@ class Bot(TelegramObject):
                 keyboard or to force a reply from the user.
             timeout (Optional[float]): If this value is specified, use it as
                 the definitive timeout (in seconds) for urlopen() operations.
-            network_delay (Optional[float]): If using the timeout (which is
-                a `timeout` for the Telegram servers operation),
-                then `network_delay` as an extra delay (in seconds) to
-                compensate for network latency. Defaults to 2.
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the
@@ -505,10 +482,6 @@ class Bot(TelegramObject):
                 keyboard or to force a reply from the user.
             timeout (Optional[float]): If this value is specified, use it as
                 the definitive timeout (in seconds) for urlopen() operations.
-            network_delay (Optional[float]): If using the timeout (which is
-                a `timeout` for the Telegram servers operation),
-                then `network_delay` as an extra delay (in seconds) to
-                compensate for network latency. Defaults to 2.
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the
@@ -562,10 +535,6 @@ class Bot(TelegramObject):
                 keyboard or to force a reply from the user.
             timeout (Optional[float]): If this value is specified, use it as
                 the definitive timeout (in seconds) for urlopen() operations.
-            network_delay (Optional[float]): If using the timeout (which is
-                a `timeout` for the Telegram servers operation),
-                then `network_delay` as an extra delay (in seconds) to
-                compensate for network latency. Defaults to 2.
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the
@@ -624,10 +593,6 @@ class Bot(TelegramObject):
                 keyboard or to force a reply from the user.
             timeout (Optional[float]): If this value is specified, use it as
                 the definitive timeout (in seconds) for urlopen() operations.
-            network_delay (Optional[float]): If using the timeout (which is
-                a `timeout` for the Telegram servers operation),
-                then `network_delay` as an extra delay (in seconds) to
-                compensate for network latency. Defaults to 2.
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the
@@ -677,10 +642,6 @@ class Bot(TelegramObject):
                 keyboard or to force a reply from the user.
             timeout (Optional[float]): If this value is specified, use it as
                 the definitive timeout (in seconds) for urlopen() operations.
-            network_delay (Optional[float]): If using the timeout (which is
-                a `timeout` for the Telegram servers operation),
-                then `network_delay` as an extra delay (in seconds) to
-                compensate for network latency. Defaults to 2.
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the
@@ -739,10 +700,6 @@ class Bot(TelegramObject):
                 keyboard or to force a reply from the user.
             timeout (Optional[float]): If this value is specified, use it as
                 the definitive timeout (in seconds) for urlopen() operations.
-            network_delay (Optional[float]): If using the timeout (which is
-                a `timeout` for the Telegram servers operation),
-                then `network_delay` as an extra delay (in seconds) to
-                compensate for network latency. Defaults to 2.
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the
@@ -800,10 +757,6 @@ class Bot(TelegramObject):
                 keyboard or to force a reply from the user.
             timeout (Optional[float]): If this value is specified, use it as
                 the definitive timeout (in seconds) for urlopen() operations.
-            network_delay (Optional[float]): If using the timeout (which is
-                a `timeout` for the Telegram servers operation),
-                then `network_delay` as an extra delay (in seconds) to
-                compensate for network latency. Defaults to 2.
 
         Returns:
             :class:`telegram.Message`: On success, instance representing the
@@ -895,10 +848,6 @@ class Bot(TelegramObject):
         Keyword Args:
             timeout (Optional[float]): If this value is specified, use it as
                 the definitive timeout (in seconds) for urlopen() operations.
-            network_delay (Optional[float]): If using the timeout (which is
-                a `timeout` for the Telegram servers operation),
-                then `network_delay` as an extra delay (in seconds) to
-                compensate for network latency. Defaults to 2.
 
         Returns:
             bool: On success, `True` is returned.
@@ -927,8 +876,7 @@ class Bot(TelegramObject):
             data['switch_pm_parameter'] = switch_pm_parameter
 
         result = request.post(url, data,
-                              timeout=kwargs.get('timeout'),
-                              network_delay=kwargs.get('network_delay'))
+                              timeout=kwargs.get('timeout'))
 
         return result
 
@@ -953,10 +901,6 @@ class Bot(TelegramObject):
         Keyword Args:
             timeout (Optional[float]): If this value is specified, use it as
                 the definitive timeout (in seconds) for urlopen() operations.
-            network_delay (Optional[float]): If using the timeout (which is
-                a `timeout` for the Telegram servers operation),
-                then `network_delay` as an extra delay (in seconds) to
-                compensate for network latency. Defaults to 2.
 
         Returns:
             list[:class:`telegram.UserProfilePhotos`]: A list of
@@ -977,8 +921,7 @@ class Bot(TelegramObject):
             data['limit'] = limit
 
         result = request.post(url, data,
-                              timeout=kwargs.get('timeout'),
-                              network_delay=kwargs.get('network_delay'))
+                              timeout=kwargs.get('timeout'))
 
         return UserProfilePhotos.de_json(result)
 
@@ -997,10 +940,6 @@ class Bot(TelegramObject):
         Keyword Args:
             timeout (Optional[float]): If this value is specified, use it as
                 the definitive timeout (in seconds) for urlopen() operations.
-            network_delay (Optional[float]): If using the timeout (which is
-                a `timeout` for the Telegram servers operation),
-                then `network_delay` as an extra delay (in seconds) to
-                compensate for network latency. Defaults to 2.
 
         Returns:
             :class:`telegram.File`: On success, a :class:`telegram.File`
@@ -1016,8 +955,7 @@ class Bot(TelegramObject):
         data = {'file_id': file_id}
 
         result = request.post(url, data,
-                              timeout=kwargs.get('timeout'),
-                              network_delay=kwargs.get('network_delay'))
+                              timeout=kwargs.get('timeout'))
 
         if result.get('file_path'):
             result['file_path'] = '%s/%s' % (self.base_file_url,
@@ -1045,10 +983,6 @@ class Bot(TelegramObject):
         Keyword Args:
             timeout (Optional[float]): If this value is specified, use it as
                 the definitive timeout (in seconds) for urlopen() operations.
-            network_delay (Optional[float]): If using the timeout (which is
-                a `timeout` for the Telegram servers operation),
-                then `network_delay` as an extra delay (in seconds) to
-                compensate for network latency. Defaults to 2.
 
         Returns:
             bool: On success, `True` is returned.
@@ -1064,8 +998,7 @@ class Bot(TelegramObject):
                 'user_id': user_id}
 
         result = request.post(url, data,
-                              timeout=kwargs.get('timeout'),
-                              network_delay=kwargs.get('network_delay'))
+                              timeout=kwargs.get('timeout'))
 
         return result
 
@@ -1089,10 +1022,6 @@ class Bot(TelegramObject):
         Keyword Args:
             timeout (Optional[float]): If this value is specified, use it as
                 the definitive timeout (in seconds) for urlopen() operations.
-            network_delay (Optional[float]): If using the timeout (which is
-                a `timeout` for the Telegram servers operation),
-                then `network_delay` as an extra delay (in seconds) to
-                compensate for network latency. Defaults to 2.
 
         Returns:
             bool: On success, `True` is returned.
@@ -1108,8 +1037,7 @@ class Bot(TelegramObject):
                 'user_id': user_id}
 
         result = request.post(url, data,
-                              timeout=kwargs.get('timeout'),
-                              network_delay=kwargs.get('network_delay'))
+                              timeout=kwargs.get('timeout'))
 
         return result
 
@@ -1158,13 +1086,11 @@ class Bot(TelegramObject):
             data['show_alert'] = show_alert
 
         result = request.post(url, data,
-                              timeout=kwargs.get('timeout'),
-                              network_delay=kwargs.get('network_delay'))
+                              timeout=kwargs.get('timeout'))
 
         return result
 
     @log
-    @message
     def editMessageText(self,
                         text,
                         chat_id=None,
@@ -1172,6 +1098,7 @@ class Bot(TelegramObject):
                         inline_message_id=None,
                         parse_mode=None,
                         disable_web_page_preview=None,
+                        reply_markup=None,
                         **kwargs):
         """Use this method to edit text messages sent by the bot or via the bot
         (for inline bots).
@@ -1194,16 +1121,12 @@ class Bot(TelegramObject):
             italic, fixed-width text or inline URLs in your bot's message.
           disable_web_page_preview:
             Disables link previews for links in this message.
+          reply_markup:
+            A JSON-serialized object for an inline keyboard.
 
         Keyword Args:
-            reply_markup (Optional[:class:`telegram.InlineKeyboardMarkup`]):
-                A JSON-serialized object for an inline keyboard.
             timeout (Optional[float]): If this value is specified, use it as
                 the definitive timeout (in seconds) for urlopen() operations.
-            network_delay (Optional[float]): If using the timeout (which is
-                a `timeout` for the Telegram servers operation),
-                then `network_delay` as an extra delay (in seconds) to
-                compensate for network latency. Defaults to 2.
 
         Returns:
             :class:`telegram.Message`: On success, if edited message is sent by
@@ -1229,8 +1152,16 @@ class Bot(TelegramObject):
             data['parse_mode'] = parse_mode
         if disable_web_page_preview:
             data['disable_web_page_preview'] = disable_web_page_preview
+        if reply_markup:
+            if isinstance(reply_markup, ReplyMarkup):
+                data['reply_markup'] = reply_markup.to_json()
+            else:
+                data['reply_markup'] = reply_markup
 
-        return url, data
+        result = request.post(url, data,
+                              timeout=kwargs.get('timeout'))
+
+        return Message.de_json(result)
 
     @log
     @message
@@ -1259,10 +1190,6 @@ class Bot(TelegramObject):
                 A JSON-serialized object for an inline keyboard.
             timeout (Optional[float]): If this value is specified, use it as
                 the definitive timeout (in seconds) for urlopen() operations.
-            network_delay (Optional[float]): If using the timeout (which is
-                a `timeout` for the Telegram servers operation),
-                then `network_delay` as an extra delay (in seconds) to
-                compensate for network latency. Defaults to 2.
 
         Returns:
             :class:`telegram.Message`: On success, if edited message is sent by
@@ -1314,10 +1241,6 @@ class Bot(TelegramObject):
                 A JSON-serialized object for an inline keyboard.
             timeout (Optional[float]): If this value is specified, use it as
                 the definitive timeout (in seconds) for urlopen() operations.
-            network_delay (Optional[float]): If using the timeout (which is
-                a `timeout` for the Telegram servers operation),
-                then `network_delay` as an extra delay (in seconds) to
-                compensate for network latency. Defaults to 2.
 
         Returns:
             :class:`telegram.Message`: On success, if edited message is sent by
@@ -1346,7 +1269,8 @@ class Bot(TelegramObject):
     def getUpdates(self,
                    offset=None,
                    limit=100,
-                   **kwargs):
+                   timeout=0,
+                   network_delay=.2):
         """Use this method to receive incoming updates using long polling.
 
         Args:
@@ -1359,14 +1283,14 @@ class Bot(TelegramObject):
           limit:
             Limits the number of updates to be retrieved. Values between 1-100
             are accepted. Defaults to 100.
-
-        Keyword Args:
-            timeout (Optional[float]): If this value is specified, use it as
-                the definitive timeout (in seconds) for urlopen() operations.
-            network_delay (Optional[float]): If using the timeout (which is
-                a `timeout` for the Telegram servers operation),
-                then `network_delay` as an extra delay (in seconds) to
-                compensate for network latency. Defaults to 2.
+          timeout:
+            Timeout in seconds for long polling. Defaults to 0, i.e. usual
+            short polling.
+          network_delay:
+            Additional timeout in seconds to allow the response from Telegram
+            to take some time when using long polling. Defaults to 2, which
+            should be enough for most connections. Increase it if it takes very
+            long for data to be transmitted from and to the Telegram servers.
 
         Returns:
             list[:class:`telegram.Message`]: A list of :class:`telegram.Update`
@@ -1379,16 +1303,16 @@ class Bot(TelegramObject):
 
         url = '{0}/getUpdates'.format(self.base_url)
 
-        data = {}
+        data = {'timeout': timeout}
 
         if offset:
             data['offset'] = offset
         if limit:
             data['limit'] = limit
 
-        result = request.post(url, data,
-                              timeout=kwargs.get('timeout'),
-                              network_delay=kwargs.get('network_delay'))
+        urlopen_timeout = timeout + network_delay
+
+        result = request.post(url, data, timeout=urlopen_timeout)
 
         if result:
             self.logger.debug(
@@ -1417,10 +1341,6 @@ class Bot(TelegramObject):
         Keyword Args:
             timeout (Optional[float]): If this value is specified, use it as
                 the definitive timeout (in seconds) for urlopen() operations.
-            network_delay (Optional[float]): If using the timeout (which is
-                a `timeout` for the Telegram servers operation),
-                then `network_delay` as an extra delay (in seconds) to
-                compensate for network latency. Defaults to 2.
 
         Returns:
             bool: On success, `True` is returned.
@@ -1440,8 +1360,7 @@ class Bot(TelegramObject):
             data['certificate'] = certificate
 
         result = request.post(url, data,
-                              timeout=kwargs.get('timeout'),
-                              network_delay=kwargs.get('network_delay'))
+                              timeout=kwargs.get('timeout'))
 
         return result
 
