@@ -28,19 +28,20 @@ from threading import Thread, Lock, current_thread, Event
 from time import sleep
 import subprocess
 from signal import signal, SIGINT, SIGTERM, SIGABRT
+from queue import Queue
+
 from telegram import Bot, TelegramError, NullHandler
 from telegram.ext import dispatcher, Dispatcher, JobQueue
 from telegram.error import Unauthorized, InvalidToken
-from telegram.utils.updatequeue import UpdateQueue
 from telegram.utils.webhookhandler import (WebhookServer, WebhookHandler)
 
 logging.getLogger(__name__).addHandler(NullHandler())
 
 
-class Updater:
+class Updater(object):
     """
     This class, which employs the Dispatcher class, provides a frontend to
-    telegram.Bot to the programmer, so they can focus on coding the bot. It's
+    telegram.Bot to the programmer, so they can focus on coding the bot. Its
     purpose is to receive the updates from Telegram and to deliver them to said
     dispatcher. It also runs in a separate thread, so the user can interact
     with the bot, for example on the command line. The dispatcher supports
@@ -81,7 +82,7 @@ class Updater:
             self.bot = bot
         else:
             self.bot = Bot(token, base_url)
-        self.update_queue = UpdateQueue()
+        self.update_queue = Queue()
         self.job_queue = JobQueue(self.bot, job_queue_tick_interval)
         self.__exception_event = Event()
         self.dispatcher = Dispatcher(self.bot, self.update_queue, workers,
@@ -110,9 +111,10 @@ class Updater:
                 False.
             bootstrap_retries (Optional[int[): Whether the bootstrapping phase
                 of the `Updater` will retry on failures on the Telegram server.
-                < 0 - retry indefinitely
-                  0 - no retries (default)
-                > 0 - retry up to X times
+
+                |   < 0 - retry indefinitely
+                |   0 - no retries (default)
+                |   > 0 - retry up to X times
 
         Returns:
             Queue: The update queue that can be filled from the main thread
@@ -177,9 +179,10 @@ class Updater:
                 is False.
             bootstrap_retries (Optional[int[): Whether the bootstrapping phase
                 of the `Updater` will retry on failures on the Telegram server.
-                < 0 - retry indefinitely
-                  0 - no retries (default)
-                > 0 - retry up to X times
+
+                |   < 0 - retry indefinitely
+                |   0 - no retries (default)
+                |   > 0 - retry up to X times
             webhook_url (Optional[str]): Explicitly specifiy the webhook url.
                 Useful behind NAT, reverse proxy, etc. Default is derived from
                 `listen`, `port` & `url_path`.
@@ -214,7 +217,7 @@ class Updater:
         cur_interval = poll_interval
         self.logger.debug('Updater thread started')
 
-        self._set_webhook(None, bootstrap_retries)
+        self._set_webhook(None, bootstrap_retries, None)
 
         while self.running:
             try:
@@ -246,12 +249,13 @@ class Updater:
 
             sleep(cur_interval)
 
-    def _set_webhook(self, webhook_url, max_retries):
+    def _set_webhook(self, webhook_url, max_retries, cert):
         retries = 0
         while 1:
             try:
                 # Remove webhook
-                self.bot.setWebhook(webhook_url=webhook_url)
+                self.bot.setWebhook(webhook_url=webhook_url,
+                                    certificate=cert)
             except (Unauthorized, InvalidToken):
                 raise
             except TelegramError:
@@ -282,43 +286,48 @@ class Updater:
                        bootstrap_retries, webhook_url):
         self.logger.debug('Updater thread started')
         use_ssl = cert is not None and key is not None
-        url_path = "/%s" % url_path
+        if not url_path.startswith('/'):
+            url_path = '/{0}'.format(url_path)
 
         # Create and start server
         self.httpd = WebhookServer((listen, port), WebhookHandler,
                                    self.update_queue, url_path)
-        if not webhook_url:
-            webhook_url = self._gen_webhook_url(listen, port, url_path,
-                                                use_ssl)
-        self._set_webhook(webhook_url, bootstrap_retries)
 
         if use_ssl:
-            # Check SSL-Certificate with openssl, if possible
-            try:
-                exit_code = subprocess.call(["openssl", "x509", "-text",
-                                             "-noout", "-in", cert],
-                                            stdout=open(os.devnull, 'wb'),
-                                            stderr=subprocess.STDOUT)
-            except OSError:
-                exit_code = 0
+            self._check_ssl_cert(cert, key)
 
-            if exit_code is 0:
-                try:
-                    self.httpd.socket = ssl.wrap_socket(self.httpd.socket,
-                                                        certfile=cert,
-                                                        keyfile=key,
-                                                        server_side=True)
-                except ssl.SSLError as error:
-                    self.logger.exception('failed to init SSL socket')
-                    raise TelegramError(str(error))
-            else:
-                raise TelegramError('SSL Certificate invalid')
+            # DO NOT CHANGE: Only set webhook if SSL is handled by library
+            if not webhook_url:
+                webhook_url = self._gen_webhook_url(listen, port, url_path)
+
+            self._set_webhook(webhook_url, bootstrap_retries,
+                              open(cert, 'rb'))
 
         self.httpd.serve_forever(poll_interval=1)
 
-    def _gen_webhook_url(self, listen, port, url_path, use_ssl):
-        return '{proto}://{listen}:{port}{path}'.format(
-            proto='https' if use_ssl else 'http',
+    def _check_ssl_cert(self, cert, key):
+        # Check SSL-Certificate with openssl, if possible
+        try:
+            exit_code = subprocess.call(["openssl", "x509", "-text",
+                                         "-noout", "-in", cert],
+                                        stdout=open(os.devnull, 'wb'),
+                                        stderr=subprocess.STDOUT)
+        except OSError:
+            exit_code = 0
+        if exit_code is 0:
+            try:
+                self.httpd.socket = ssl.wrap_socket(self.httpd.socket,
+                                                    certfile=cert,
+                                                    keyfile=key,
+                                                    server_side=True)
+            except ssl.SSLError as error:
+                self.logger.exception('Failed to init SSL socket')
+                raise TelegramError(str(error))
+        else:
+            raise TelegramError('SSL Certificate invalid')
+
+    def _gen_webhook_url(self, listen, port, url_path):
+        return 'https://{listen}:{port}{path}'.format(
             listen=listen,
             port=port,
             path=url_path)
@@ -383,7 +392,12 @@ class Updater:
 
     def signal_handler(self, signum, frame):
         self.is_idle = False
-        self.stop()
+        if self.running:
+            self.stop()
+        else:
+            self.logger.warning('Exiting immediately!')
+            import os
+            os._exit(1)
 
     def idle(self, stop_signals=(SIGINT, SIGTERM, SIGABRT)):
         """
