@@ -2,7 +2,8 @@
 # pylint: disable=R0902,R0912,R0913
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015 Leandro Toledo de Souza <leandrotoeldodesouza@gmail.com>
+# Copyright (C) 2015-2016
+# Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser Public License as published by
@@ -17,13 +18,14 @@
 # You should have received a copy of the GNU Lesser Public License
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
 
-"""This module contains a object that represents a Telegram Message"""
+"""This module contains a object that represents a Telegram Message."""
 
 from datetime import datetime
 from time import mktime
 
-from telegram import (Audio, Contact, Document, GroupChat, Location, PhotoSize,
-                      Sticker, TelegramObject, User, Video, Voice)
+from telegram import (Audio, Contact, Document, Chat, Location, PhotoSize,
+                      Sticker, TelegramObject, User, Video, Voice, Venue,
+                      MessageEntity)
 
 
 class Message(TelegramObject):
@@ -49,18 +51,29 @@ class Message(TelegramObject):
         caption (str):
         contact (:class:`telegram.Contact`):
         location (:class:`telegram.Location`):
-        new_chat_participant (:class:`telegram.User`):
-        left_chat_participant (:class:`telegram.User`):
+        new_chat_member (:class:`telegram.User`):
+        left_chat_member (:class:`telegram.User`):
         new_chat_title (str):
         new_chat_photo (List[:class:`telegram.PhotoSize`]):
         delete_chat_photo (bool):
         group_chat_created (bool):
+        supergroup_chat_created (bool):
+        migrate_to_chat_id (int):
+        migrate_from_chat_id (int):
+        channel_chat_created (bool):
+
+    Deprecated: 4.0
+        new_chat_participant (:class:`telegram.User`): Use `new_chat_member`
+        instead.
+
+        left_chat_participant  (:class:`telegram.User`): Use `left_chat_member`
+        instead.
 
     Args:
         message_id (int):
         from_user (:class:`telegram.User`):
         date (:class:`datetime.datetime`):
-        chat (:class:`telegram.User` or :class:`telegram.GroupChat`):
+        chat (:class:`telegram.Chat`):
         **kwargs: Arbitrary keyword arguments.
 
     Keyword Args:
@@ -77,12 +90,16 @@ class Message(TelegramObject):
         caption (Optional[str]):
         contact (Optional[:class:`telegram.Contact`]):
         location (Optional[:class:`telegram.Location`]):
-        new_chat_participant (Optional[:class:`telegram.User`]):
-        left_chat_participant (Optional[:class:`telegram.User`]):
+        new_chat_member (Optional[:class:`telegram.User`]):
+        left_chat_member (Optional[:class:`telegram.User`]):
         new_chat_title (Optional[str]):
         new_chat_photo (Optional[List[:class:`telegram.PhotoSize`]):
         delete_chat_photo (Optional[bool]):
         group_chat_created (Optional[bool]):
+        supergroup_chat_created (Optional[bool]):
+        migrate_to_chat_id (Optional[int]):
+        migrate_from_chat_id (Optional[int]):
+        channel_chat_created (Optional[bool]):
     """
 
     def __init__(self,
@@ -101,6 +118,7 @@ class Message(TelegramObject):
         self.forward_date = kwargs.get('forward_date')
         self.reply_to_message = kwargs.get('reply_to_message')
         self.text = kwargs.get('text', '')
+        self.entities = kwargs.get('entities', list())
         self.audio = kwargs.get('audio')
         self.document = kwargs.get('document')
         self.photo = kwargs.get('photo')
@@ -110,12 +128,20 @@ class Message(TelegramObject):
         self.caption = kwargs.get('caption', '')
         self.contact = kwargs.get('contact')
         self.location = kwargs.get('location')
-        self.new_chat_participant = kwargs.get('new_chat_participant')
-        self.left_chat_participant = kwargs.get('left_chat_participant')
+        self.venue = kwargs.get('venue')
+        self.new_chat_member = kwargs.get('new_chat_member')
+        self.left_chat_member = kwargs.get('left_chat_member')
         self.new_chat_title = kwargs.get('new_chat_title', '')
         self.new_chat_photo = kwargs.get('new_chat_photo')
         self.delete_chat_photo = bool(kwargs.get('delete_chat_photo', False))
         self.group_chat_created = bool(kwargs.get('group_chat_created', False))
+        self.supergroup_chat_created = bool(kwargs.get(
+            'supergroup_chat_created', False))
+        self.migrate_to_chat_id = int(kwargs.get('migrate_to_chat_id', 0))
+        self.migrate_from_chat_id = int(kwargs.get('migrate_from_chat_id', 0))
+        self.channel_chat_created = bool(kwargs.get('channel_chat_created',
+                                                    False))
+        self.pinned_message = kwargs.get('pinned_message')
 
     @property
     def chat_id(self):
@@ -126,7 +152,7 @@ class Message(TelegramObject):
     def de_json(data):
         """
         Args:
-            data (str):
+            data (dict):
 
         Returns:
             telegram.Message:
@@ -136,38 +162,25 @@ class Message(TelegramObject):
 
         data['from_user'] = User.de_json(data.get('from'))
         data['date'] = datetime.fromtimestamp(data['date'])
-        if 'first_name' in data.get('chat', ''):
-            data['chat'] = User.de_json(data.get('chat'))
-        elif 'title' in data.get('chat', ''):
-            data['chat'] = GroupChat.de_json(data.get('chat'))
-        data['forward_from'] = \
-            User.de_json(data.get('forward_from'))
-        data['forward_date'] = \
-            Message._fromtimestamp(data.get('forward_date'))
+        data['chat'] = Chat.de_json(data.get('chat'))
+        data['entities'] = MessageEntity.de_list(data.get('entities'))
+        data['forward_from'] = User.de_json(data.get('forward_from'))
+        data['forward_date'] = Message._fromtimestamp(data.get('forward_date'))
         data['reply_to_message'] = \
             Message.de_json(data.get('reply_to_message'))
-        data['audio'] = \
-            Audio.de_json(data.get('audio'))
-        data['document'] = \
-            Document.de_json(data.get('document'))
-        data['photo'] = \
-            PhotoSize.de_list(data.get('photo'))
-        data['sticker'] = \
-            Sticker.de_json(data.get('sticker'))
-        data['video'] = \
-            Video.de_json(data.get('video'))
-        data['voice'] = \
-            Voice.de_json(data.get('voice'))
-        data['contact'] = \
-            Contact.de_json(data.get('contact'))
-        data['location'] = \
-            Location.de_json(data.get('location'))
-        data['new_chat_participant'] = \
-            User.de_json(data.get('new_chat_participant'))
-        data['left_chat_participant'] = \
-            User.de_json(data.get('left_chat_participant'))
-        data['new_chat_photo'] = \
-            PhotoSize.de_list(data.get('new_chat_photo'))
+        data['audio'] = Audio.de_json(data.get('audio'))
+        data['document'] = Document.de_json(data.get('document'))
+        data['photo'] = PhotoSize.de_list(data.get('photo'))
+        data['sticker'] = Sticker.de_json(data.get('sticker'))
+        data['video'] = Video.de_json(data.get('video'))
+        data['voice'] = Voice.de_json(data.get('voice'))
+        data['contact'] = Contact.de_json(data.get('contact'))
+        data['location'] = Location.de_json(data.get('location'))
+        data['venue'] = Venue.de_json(data.get('venue'))
+        data['new_chat_member'] = User.de_json(data.get('new_chat_member'))
+        data['left_chat_member'] = User.de_json(data.get('left_chat_member'))
+        data['new_chat_photo'] = PhotoSize.de_list(data.get('new_chat_photo'))
+        data['pinned_message'] = Message.de_json(data.get('pinned_message'))
 
         return Message(**data)
 
@@ -192,6 +205,8 @@ class Message(TelegramObject):
             data['forward_date'] = self._totimestamp(self.forward_date)
         if self.photo:
             data['photo'] = [p.to_dict() for p in self.photo]
+        if self.entities:
+            data['entities'] = [e.to_dict() for e in self.entities]
         if self.new_chat_photo:
             data['new_chat_photo'] = [p.to_dict() for p in self.new_chat_photo]
 
