@@ -19,20 +19,12 @@
 
 """This module contains functions to validate function arguments"""
 
-try:
-    type(basestring)
-except NameError:
-    basestring = str
+from telegram.error import InvalidToken
 
 
-def validate_string(arg, name):
-    """
-    Validate a string argument. Raises a ValueError if `arg` is neither an
-    instance of basestring (Python 2) or str (Python 3) nor None.
-
-    Args:
-        arg (basestring): The value to be tested
-        name (str): The name of the argument, for the error message
-    """
-    if not isinstance(arg, basestring) and arg is not None:
-        raise ValueError(name + ' is not a string')
+def validate_token(token):
+    """a very basic validation on token"""
+    left, sep, _right = token.partition(':')
+    if (not sep) or (not left.isdigit()) or (len(left) < 3):
+        raise InvalidToken()
+    return token
